'''Program to create a ghost particle in the center of a ring
and do the rdf between ring centers'''
#NOTE The number of frames and bin size affect the
#peak heights. I'm not sure how to account for this.
#NOTE: If this is used more it'd be nice to incorporate the autocorr.

import sys
import mdtraj as md
import numpy as np
#import cme_utils
import matplotlib.pyplot as plt
import math

TOP_FILE = "restart.hoomdxml" #sys.argv[1]
TRAJ_FILE = "traj.dcd" #sys.argv[2]
t = md.load(TRAJ_FILE, top = TOP_FILE)

numatoms = 20
print "You are calculating for %d atoms within a molecule" %numatoms 
nummol = int(t.n_atoms/numatoms)
axes = t.unitcell_lengths[t.n_frames-1]

#Empty lists
p1_positions = []
p2_positions = []
p3_positions = []
center_positions = []
distance_list = []
dbin = []
rdf = []

def pbc(vec):
    """pbc is to account for the periodic boundry conditions."""
    for i,v in enumerate(vec):
        if v>axes[i]/2.:
            vec[i]=v-axes[i]
        if v<-axes[i]/2.:
            vec[i]=v+axes[i]
    return vec

def average_position(p1, p2, p3):
    "Calculate the average position from the center ring"""
    v1 = p3 - p1
    v2 = p3 - p2
    v1 = pbc(v1)
    v2 = pbc(v2)
    p1 = p3 - v1
    p2 = p3 - v2
    average = np.mean((p1, p2, p3), axis = 0)
    return average

def distances(vec1, vec2):
    """Make the point of interest the center of the box
    No other atoms can be greater than l/2 away."""
    diff = vec1 - vec2
    for i in range(3):
        if diff[i] > axes[i]/2.:
            diff[i]= diff[i] - axes[i]
        if diff[i] < -axes[i]/2.:
            diff[i]= diff[i] + axes[i]
    dist = np.linalg.norm(diff)
    return dist

"""indices"""
for m in range(t.n_frames-24, t.n_frames):#Loop to get frames of interest.
    p1_positions = t.xyz[m,4:-1:numatoms,:]
    p2_positions = t.xyz[m,8:-1:numatoms,:]
    p3_positions = t.xyz[m,11:-1:numatoms,:]
    for i in range(nummol):
        center_positions.append(average_position(p1_positions[i], p2_positions[i], p3_positions[i]))
    center_positions = np.array(center_positions)
    """Deleting and remaking position arrays for the loop.
    Is there a better way to do this?"""
    del p1_positions, p2_positions, p3_positions
    p1_positions = []
    p2_positions = []
    p3_positions = []

    size = len(center_positions)
    for i in range(size-1):
        for j in range(i,size):
            distance_list.append(distances(center_positions[i], center_positions[j]))
    """Delete and remake for loop."""
    del center_positions
    center_positions = []

distance_list = np.array(distance_list)

bins = 200 #Set up number of bins.
rho = (200.)/(axes[0]**3)
'''Density rho is based off of number of particles in box. 
This is currently N particles.''' 
distrib = np.histogram(distance_list, bins)
for i in range(len(distrib[0])):
    start = 0.0
    end = distrib[1][i]
    dr = end - start
    mid = (start+end)/2.
    point = distrib[0][i]/(24*rho*4.*math.pi*mid*dr)#24 is the number of frames
    start = distrib[1][i]
    rdf.append([end, point])
    #start = start + i*dr
    #end = end + i*dr

rdf = np.array(rdf)

record_a = True
record_b = True
for i, line in enumerate(rdf):
    val = line[0]*10.
    if val <= 1.75:
        a = i
        record_a = False
    if val <= 2.6:
        b = i
        record_b = False
    if val >= 3.6:
        c = i
        break

max1 = np.amax(rdf[5:a][:,1])
max2 = np.amax(rdf[a:b][:,1])
max3 = np.amax(rdf[b:c][:,1])

for i in range(len(rdf)):
    if rdf[i,1] == max1:
        P1 = [rdf[i,0], rdf[i,1]]
    if rdf[i,1] == max2:
        P2 = [rdf[i,0], rdf[i,1]]
    if rdf[i,1] == max3:
        P3 = rdf[i,0], rdf[i,1]
#print P1, P2, P3

for i in rdf:
    print i[0], i[1]
#plt.hist(distance_list, 100)

plt.plot(10.*rdf[:,0],rdf[:,1])
plt.title('radial distribution function')
plt.ylabel('g(r)')
plt.xlabel('r$(\sigma)$')
plt.xlim([0.3,6])
#plt.xlim([0.3,10.*axes[1]])
plt.axhspan(1,1, facecolor = '0.2', alpha = 1)
plt.savefig('manualrdf.pdf')
plt.show()
